package com.qmth.cqb.paper.service.export;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import main.java.com.UpYun;

import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;
import cn.com.qmth.examcloud.common.uac.entity.AccessUser;
import cn.com.qmth.examcloud.common.util.excel.ExcelWriter;

import com.qmth.cqb.paper.dto.ObjectiveQuestionStructure;
import com.qmth.cqb.paper.dto.PaperDetailExp;
import com.qmth.cqb.paper.dto.PaperDetailUnitExp;
import com.qmth.cqb.paper.dto.PaperExp;
import com.qmth.cqb.paper.dto.SubjectiveQuestionStructure;
import com.qmth.cqb.paper.model.ExamFile;
import com.qmth.cqb.paper.model.ExportStructure;
import com.qmth.cqb.paper.model.ExtractConfig;
import com.qmth.cqb.paper.model.QuestionTypeNum;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.enums.ExamFileType;
import com.qmth.cqb.utils.word.DocxProcessUtil;

/**
 * @author  	chenken
 * @date    	2017年7月7日 上午11:29:49
 * @company 	QMTH
 * @description 陕西师范试卷导出service
 */
@Service("sxsfExportPaperService")
public class SxsfExportPaperService extends ExportPaperAbstractService {
	
    @Override
    public void downloadPaper(String paperId, HttpServletResponse response) throws Exception {
        Map<String,Object> dataMap = initExportPaper(paperId);
        List<String> fileList = new ArrayList<String>();
        if (dataMap.get("fileName") != null) {
            String paperfileName = (String) dataMap.get("fileName")+DOCX_SUFFIX;
            String answerFileName = (String) dataMap.get("fileName")+"_"+ExamFileType.ANSWER.getName()+DOCX_SUFFIX;
            DocxProcessUtil.exportWord(dataMap, paperfileName,SXSF_TEMPLATE_PAPER);
            DocxProcessUtil.exportWord(dataMap,answerFileName,SXSF_TEMPLATE_ANSWER);
            DocxProcessUtil.processImage(paperfileName,getPkgList(paperId));
            DocxProcessUtil.processImage(answerFileName,getPkgList(paperId));
            fileList.add(paperfileName);
            fileList.add(answerFileName);
            DocxProcessUtil.processDownload(fileList, response);
        }
    }

    @Override
    public void uploadFile(ExtractConfig extractConfig,String paperId,AccessUser accessUser,Map<String,Object> exportStructureMap) throws Exception{
    	Map<String,Object> dataMap = initExportPaper(paperId);
        if (dataMap.get("fileName") != null) {
        	String currNum = CommonUtils.getCurNum();
    		ExportStructure exportStructure = (ExportStructure) exportStructureMap.get("exportStructure");
    		List<QuestionTypeNum> questionTypeNums = new ArrayList<QuestionTypeNum>();
    		if(exportStructure!=null){
    			questionTypeNums = exportStructure.getQuestionTypeNums();
    		}
    		PaperExp paperExp = (PaperExp) dataMap.get("paper");
        	List<PaperDetailExp> objectiveDetails = paperExp.getObjectiveDetails();
        	//检查客观题的数量是否大于试卷导出设置中的客观题数量
        	if(questionTypeNums.size()>0){
        		checkObjectiveDetailsNum(paperExp,objectiveDetails,questionTypeNums);
        	}
    		//上传试卷
        	uploadPaperWord(dataMap,extractConfig,paperId,accessUser,currNum);
        	//上传答案
        	uploadAnswerWord(dataMap,extractConfig,paperId,accessUser,currNum);
        	//上传试卷结构
        	if(questionTypeNums.size()>0){
        		uploadPaperStructure(dataMap,extractConfig,paperId,accessUser,currNum,questionTypeNums);
        	}
        }
    }
    
	/**
     * 生成试卷Word，上传至又拍云
     * @param orgName
     * @param dataMap
     * @param extractConfig
     * @param paperId
     * @param accessUser
     */
    private void uploadPaperWord(Map<String,Object> dataMap,ExtractConfig extractConfig,String paperId,AccessUser accessUser,String currNum){
    	String paperfileName = currNum+ExamFileType.PAPER.name()+DOCX_SUFFIX;
    	try {
			DocxProcessUtil.exportWord(dataMap,paperfileName,SXSF_TEMPLATE_PAPER);
			DocxProcessUtil.processImage(paperfileName,getPkgList(paperId));
			File paperFile  = new File(TEMP_FILE_EXP+paperfileName);
			String paperFilePath = uploadUrl+paperfileName;
            UpYun upyun = new UpYun(bucketName,userName,password);
			upyun.writeFile(paperFilePath, paperFile,true);
			examFileService.saveExamFile(new ExamFile(extractConfig,paperFilePath,ExamFileType.PAPER,DOCX_SUFFIX),accessUser);
			paperFile.delete();
		} catch (Exception e) {
			e.printStackTrace();
		}
    }
    
    /**
     * 生成答案Word，上传至又拍云
     * @param orgName
     * @param dataMap
     * @param extractConfig
     * @param paperId
     * @param accessUser
     */
    private void uploadAnswerWord(Map<String,Object> dataMap,ExtractConfig extractConfig,String paperId,AccessUser accessUser,String currNum){
    	String answerFileName = currNum+ExamFileType.ANSWER.name()+DOCX_SUFFIX;
    	try {
    		DocxProcessUtil.exportWord(dataMap,answerFileName,SXSF_TEMPLATE_ANSWER);
    		DocxProcessUtil.processImage(answerFileName,getPkgList(paperId));
    		File answerFile  = new File(TEMP_FILE_EXP+answerFileName);
    		String answerFilePath = uploadUrl+answerFileName;
            UpYun upyun = new UpYun(bucketName,userName,password);
			upyun.writeFile(answerFilePath, answerFile,true);
			examFileService.saveExamFile(new ExamFile(extractConfig,answerFilePath,ExamFileType.ANSWER,DOCX_SUFFIX),accessUser);
			answerFile.delete();
		} catch (Exception e) {
			e.printStackTrace();
		}
    }
    
    /**
     * 生成试卷结构，上传至又拍云
     */
    private void uploadPaperStructure(Map<String,Object> dataMap,ExtractConfig extractConfig,String paperId,AccessUser accessUser,String currNum,List<QuestionTypeNum> questionTypeNums) {
    	exportObjectiveQuestionStructures(dataMap,extractConfig,accessUser,currNum,questionTypeNums);
    	exportSubjectiveQuestionStructures(dataMap,extractConfig,accessUser,currNum);
	}
    
    /**
     * 获得客观题导出结构，并上传至又拍云
     * @param paperExp
     * @return
     */
    private void exportObjectiveQuestionStructures(Map<String,Object> dataMap,ExtractConfig extractConfig,AccessUser accessUser,String currNum,List<QuestionTypeNum> questionTypeNums){
    	String objectiveFilename = currNum+ExamFileType.PAPER_STRUCTURE_OBJECTIVE.name()+EXCEL_SUFFIX;
    	PaperExp paperExp = (PaperExp) dataMap.get("paper");
    	List<PaperDetailExp> objectiveDetails = paperExp.getObjectiveDetails();
    	//根据试卷结构导出设置中的数量补齐客观题
    	fillObjectiveQuestions(objectiveDetails,questionTypeNums);
    	List<ObjectiveQuestionStructure> objectiveQuestionStructureList = new ArrayList<ObjectiveQuestionStructure>();
    	for(PaperDetailExp paperDetailExp:objectiveDetails){
    		for(PaperDetailUnitExp unit:paperDetailExp.getPaperDetailUnits()){
    			objectiveQuestionStructureList.add(new ObjectiveQuestionStructure(paperExp,paperDetailExp,unit));
    		}
    	}
    	ExcelWriter objectiveExcelExporter = new ExcelWriter(ObjectiveQuestionStructure.class); 
		try {
			File file = new File(TEMP_FILE_EXP+objectiveFilename);
			FileOutputStream out = new FileOutputStream(file);
			objectiveExcelExporter.write(objectiveFilename,objectiveQuestionStructureList,out);
			String objectiveFilePath = uploadUrl+objectiveFilename;
            UpYun upyun = new UpYun(bucketName,userName,password);
			upyun.writeFile(objectiveFilePath,file,true);
			examFileService.saveExamFile(new ExamFile(extractConfig,objectiveFilePath,ExamFileType.PAPER_STRUCTURE_OBJECTIVE,EXCEL_SUFFIX),accessUser);
			file.delete();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
    }
    
    /**
     * 根据试卷结构导出设置中的数量补齐客观题
     * @param objectiveDetails
     * @param questionTypeNums
     */
    public void fillObjectiveQuestions(List<PaperDetailExp> objectiveDetails,List<QuestionTypeNum> questionTypeNums) {
    	for(PaperDetailExp paperDetailExp:objectiveDetails){
    		List<PaperDetailUnitExp> paperDetailUnits = paperDetailExp.getPaperDetailUnits();
    		//1.将大题中的小题排序
    		Collections.sort(paperDetailUnits);
    		//2.将小题的number重新设置
    		for(int i = 0;i<paperDetailUnits.size();i++){
    			PaperDetailUnitExp paperDetailUnitExp = paperDetailUnits.get(i);
    			paperDetailUnitExp.setNumber(i+1);
    		}
    		//3.根据试卷结构导出设置中的数量补齐客观题
    		QuesStructType type = paperDetailUnits.get(0).getQuestionType();
    		int unitSize = paperDetailUnits.size();
    		for(QuestionTypeNum questionTypeNum:questionTypeNums){
    			if(type==questionTypeNum.getQuestionType()){
    				int length = questionTypeNum.getQuantity()-unitSize;
    				for(int i = 0;i<length;i++){
    					PaperDetailUnitExp paperDetailUnitExp = new PaperDetailUnitExp();
    					paperDetailUnitExp.setQuestionType(type);
    					paperDetailUnitExp.setScore((double) 0);
    					paperDetailUnitExp.setNumber(unitSize+(i+1));
    					Question question = new Question();
    					question.setQuesAnswer("#");
    					paperDetailUnitExp.setQuestion(question);
    					paperDetailUnits.add(paperDetailUnitExp);
    				}
    			}
    		}
    	}
	}

	/**
     * 检查客观题数量是否小于试卷结构导出设置的数量
     * @param paperExp
     * @param objectiveDetails
     * @param questionTypeNums
     * @return
     */
    private void checkObjectiveDetailsNum(PaperExp paperExp,List<PaperDetailExp> objectiveDetails,List<QuestionTypeNum> questionTypeNums) {
	   for(PaperDetailExp paperDetailExp:objectiveDetails){
			for(QuestionTypeNum typeNum:questionTypeNums){
				QuesStructType quesStructType = paperDetailExp.getPaperDetailUnits().get(0).getQuestionType();
				if(typeNum.getQuestionType()==quesStructType){
					if(paperDetailExp.getUnitCount()>typeNum.getQuantity()){
						throw new RuntimeException("试卷:"+paperExp.getName()
								+"中"+quesStructType.getName()+"的数量："+paperDetailExp.getUnitCount()
								+ ",大于试卷结构导出设置的数量："+typeNum.getQuantity()+",不符合试卷结构导出规则");
					}
				}
			}
		}
    }

   	/**
     * 获得主观题导出结构，并上传至又拍云
     * @param paperExp
     * @return
     */
    private void exportSubjectiveQuestionStructures(Map<String,Object> dataMap,ExtractConfig extractConfig,AccessUser accessUser,String currNum){
    	String subjectiveFileName = currNum+ExamFileType.PAPER_STRUCTURE_SUBJECTIVE.name()+EXCEL_SUFFIX;
    	PaperExp paperExp = (PaperExp) dataMap.get("paper");
    	List<PaperDetailExp> subjectiveDetails = paperExp.getSubjectiveDetails();
    	List<SubjectiveQuestionStructure> subjectiveQuestionStructureList = new ArrayList<SubjectiveQuestionStructure>();
    	for(PaperDetailExp paperDetailExp:subjectiveDetails){
    		for(PaperDetailUnitExp unit:paperDetailExp.getPaperDetailUnits()){
    			subjectiveQuestionStructureList.add(new SubjectiveQuestionStructure(paperExp,paperDetailExp,unit));
    		}
    	}
    	ExcelWriter subjectiveExcelExporter = new ExcelWriter(SubjectiveQuestionStructure.class); 
		try {
			File file = new File(TEMP_FILE_EXP+subjectiveFileName);
			FileOutputStream out = new FileOutputStream(file);
			subjectiveExcelExporter.write(subjectiveFileName,subjectiveQuestionStructureList,out);
			String subjectiveFilePath = uploadUrl+subjectiveFileName;
            UpYun upyun = new UpYun(bucketName,userName,password);
			upyun.writeFile(subjectiveFilePath,file,true);
			examFileService.saveExamFile(new ExamFile(extractConfig,subjectiveFilePath,ExamFileType.PAPER_STRUCTURE_SUBJECTIVE,EXCEL_SUFFIX),accessUser);
			file.delete();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
    }
    
}

